# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : fisheye_correction.py
# Copyright       : 2015-2020 SORACOM, INC.
# ======================================================

import json
import logging
from logging import getLogger

import cv2
import numpy

from soracom_harvest_files import SoracomHarvestFiles

FORMAT = '%(levelname)s %(asctime)s %(funcName)s %(filename)s %(lineno)d %(message)s'
logging.basicConfig(level=logging.DEBUG, format=FORMAT)
logger = getLogger(__name__)


class FisheyeCorrection():

    def __init__(self):
        self.__harvest_files = SoracomHarvestFiles()
        self.__correction_flg = False
        self.__camera_mat = None
        self.__dist_coef = None

    def __get_json_harvest(self, name):
        "download a callibration file"

        logger.info('download %s', name)
        response = self.__harvest_files.downloadFile(name)
        if response is not None:
            json_object = json.loads(response.text)
            camera_mat = numpy.asarray(
                json_object['camera_mat'], dtype='float64')
            dist_coef = numpy.asarray(
                json_object['dist_coef'], dtype='float64')
            return camera_mat, dist_coef
        logger.error('failed download %s', name)
        return None, None

    def update_calibration(self, file_name):
        "update parameter for fisheye correction"

        camera_mat = None
        dist_coef = None
        if file_name:
            camera_mat, dist_coef = self.__get_json_harvest(file_name)
        else:
            logger.info('file_name is error.')

        if camera_mat is not None and dist_coef is not None:
            if camera_mat.shape == (3, 3) and dist_coef.shape == (1, 5):
                self.__correction_flg = True
                self.__camera_mat = camera_mat
                self.__dist_coef = dist_coef
                logger.info(
                    '%s \n %s',
                    str(self.__camera_mat),
                    str(self.__dist_coef))
            else:
                logger.error('wrong parameter.')

    def correction(self, image):
        if self.__correction_flg:
            logger.info('fish eye correction start')
            return cv2.undistort(image, self.__camera_mat, self.__dist_coef)
        logger.info('fish eye correction end')
        return image
