# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_data_upload.py
# Copyright       : 2015-2020 SORACOM, INC.
# ======================================================

import json
from datetime import datetime
from time import sleep
import traceback
from logging import getLogger
import cv2
import numpy
from soracom_harvest_files import SoracomHarvestFiles
from soracom_harvest_data import SoracomHarvestData
from soracom_data_upload_path_util import SoracomDataUploadPathUtil

logger = getLogger(__name__)

# Harvest Files and Harvest Data
HARVEST_FILES = SoracomHarvestFiles()
HARVEST_DATA = SoracomHarvestData()

class numEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, numpy.integer):
            return int(obj)
        elif isinstance(obj, numpy.floating):
            return float(obj)
        elif isinstance(obj, numpy.ndarray):
            return obj.tolist()
        else:
            return super(numEncoder, self).default(obj)


class SoracomDataUpload():
    def __init__(self):
        pass

    def uploadData(self, device_id, timestamp, data_type, images, data,
                   data_id=-1, resources=None, options=None):
        logger.info("########### SoracomDataUpload.uploadData() \
            device_id: %d data_type: %s resources: %s options: %s images: %d data: %d", \
                data_id, data_type, resources, options, len(images), len(data))
        logger.debug("uploadData options=%s", options)
        result = None
        try:
            # image path
            harvest_file_upload_paths = []
            option_image = options.get('image', None)
            debug = options.get('debug', False)
            if option_image is not None:
                index = 0
                for image in images:
                    option_image_content_type = option_image.get('contentType', None)
                    option_image_image_convert = option_image.get('imageConvert', False)
                    option_image_quality = option_image.get('quality', 80)
                    option_image_top_dir_name = option_image.get('topDirName', '')
                    option_image_filename_prefix = option_image.get('filenamePrefix', '')
                    option_image_filename_suffix = option_image.get('filenameSuffix', '')
                    option_image_extention = option_image.get('extention', '')
                    option_image_dir_type = option_image.get('dirType', 'default')
                    option_image_filename_type = option_image.get('filenameType', 'default')

                    attr = None
                    #if index > 0:
                    if len(images) > 1:
                        attr = '_'
                        attr += str(index)
                    soracom_data_upload_path_util_image = SoracomDataUploadPathUtil(
                        option_image_top_dir_name, device_id, option_image_filename_prefix,
                        option_image_filename_suffix, option_image_extention, timestamp,
                        dir_type=option_image_dir_type, filename_type=option_image_filename_type)
                    im_data_byte = None
                    if option_image_image_convert:
                        if option_image_content_type == 'image/jpeg':
                            # convert to jpeg
                            encode_param = [cv2.IMWRITE_JPEG_QUALITY, option_image_quality]
                            im_result, im_data = cv2.imencode('.jpeg', image, encode_param)
                            im_data_byte = im_data.tobytes()
                            logger.info('jpeg im_result = %d', im_result)
                            logger.info('jpeg im_data len = %d', len(im_data))
                        elif option_image_content_type == 'image/webp':
                            # convert to webp
                            encode_param = [cv2.IMWRITE_WEBP_QUALITY, option_image_quality]
                            im_result, im_data = cv2.imencode('.webp', image, encode_param)
                            im_data_byte = im_data.tobytes()
                            logger.info('webp im_result = %d', im_result)
                            logger.info('webp im_data len = %d', len(im_data))
                        else:
                            im_data_byte = image
                    else:
                        im_data_byte = image
                    full_path = soracom_data_upload_path_util_image.getFullPath(attr, int(timestamp))
                    logger.info('image full_path = %s', full_path)
                    response = HARVEST_FILES.uploadData(full_path, im_data_byte, option_image_content_type, debug)
                    harvest_file_upload_paths.append(full_path)
                    if response is not None:
                        if response.status_code != 200 and response.status_code != 201:
                            result = {}
                            result['error'] = 'images'
                            result['response'] = response
                            result['index'] = index
                            return result
                    else:
                        result = {}
                        result['error'] = 'images'
                        result['index'] = index
                        return result
                    index += 1

            option_data = options.get('data', None)
            if option_data is not None:
                index = 0
                for dt in data:
                    option_data_upload = option_data.get('upload', None)
                    logger.info('option_data_upload: %s', option_data_upload)
                    if option_data_upload is not None:
                        #Upload
                        option_data_image_file_path = option_data.get('image_file_path', None)
                        logger.info('index: %s option_data_image_file_path: %s data: %s', index, option_data_image_file_path, dt)
                        if option_data_image_file_path is not None and \
                            len(harvest_file_upload_paths) > index:
                            dt[option_data_image_file_path] = harvest_file_upload_paths[index]
                        data_byte = None
                        if option_data_upload == 'Harvest Data':
                            data_byte = json.dumps(dt, cls=numEncoder)
                            logger.info('HARVEST_DATA.uploadData %s', data_byte)
                            HARVEST_DATA.uploadData(data_byte, timestamp + float(index/1000), debug)
                        elif option_data_upload == 'Harvest Files':
                            option_data_content_type = option_data.get('contentType', None)
                            option_data_top_dir_name = option_data.get('topDirName', '')
                            option_data_filename_prefix = option_data.get('filenamePrefix', '')
                            option_data_filename_suffix = option_data.get('filenameSuffix', '')
                            option_data_extention = option_data.get('extention', '')
                            option_data_dir_type = option_data.get('dirType', 'default')
                            option_data_filename_type = option_data.get('filenameType', 'default')
                            attr = None
                            if len(data) > 1:
                                attr = '_'
                                attr += str(index)
                            soracom_data_upload_path_util_data = SoracomDataUploadPathUtil(
                                option_data_top_dir_name, device_id, option_data_filename_prefix,
                                option_data_filename_suffix, option_data_extention, timestamp,
                                dir_type=option_data_dir_type, filename_type=option_data_filename_type)
                            if option_data_content_type == 'application/json':
                                data_byte = json.dumps(dt, cls=numEncoder)
                            else:
                                data_byte = dt
                            full_path = soracom_data_upload_path_util_data.getFullPath(attr, int(timestamp))
                            logger.info('data full_path: %s', full_path)
                            response = HARVEST_FILES.uploadData(full_path, data_byte, option_data_content_type)
                            if response is not None:
                                if response.status_code != 200 and response.status_code != 201:
                                    result = {}
                                    result['error'] = 'data'
                                    result['response'] = response
                                    result['index'] = index
                                    return result
                            else:
                                result = {}
                                result['error'] = 'data'
                                result['index'] = index
                                return result
                    index += 1
        except Exception as err:
            logger.error(traceback.format_exc())
            logger.error(err)
            result = {}
            result['error'] = 'exception'
            result['exception'] = err
        return result

SORACOM_DATA_UPLOAD = None

def run(device_id, timestamp, dataType, images, data, data_id, resources, options):
    logger.info("SoracomDataUpload.run()")

    global SORACOM_DATA_UPLOAD
    if SORACOM_DATA_UPLOAD is None:
        SORACOM_DATA_UPLOAD = SoracomDataUpload()

    result = SORACOM_DATA_UPLOAD.uploadData(
        device_id, timestamp, dataType, images, data, data_id, resources, options)
    return result
