# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_data_upload_path_util.py
# Copyright       : 2015-2020 SORACOM, INC.
# ======================================================

import os
from logging import getLogger
import time
import datetime

logger = getLogger(__name__)

class SoracomDataUploadPathUtil:
    def __init__(self, top_dir_name='CameraApp0', device_id='', \
                filename_prefix='', filename_suffix='', extention='',\
                timestamp=time.time(), dir_type='default', filename_type='default'):
        self.top_dir_name = top_dir_name
        self.device_id = device_id
        self.filename_prefix = filename_prefix
        self.filename_suffix = filename_suffix
        self.extention = extention
        self.option = None
        self.dir_type = dir_type
        self.filename_type = filename_type
        self.timestamp = timestamp

    def setTimestamp(self, timestamp):
        self.timestamp = timestamp

    def setOption(self, option):
        self.option = option

    def getOption(self):
        return self.option

    def getDirPath(self, timestamp=None):
        if timestamp is not None:
            self.setTimestamp(timestamp)
        dir_path = '/'
        if self.dir_type == 'default' or self.dir_type == 'device_divideDate':
            dir_path = os.path.join(dir_path, self.top_dir_name)
            dir_path = os.path.join(dir_path, self.device_id)
            dir_path = os.path.join(dir_path, self.timestamp.strftime('%Y/%m/%d'))
        elif self.dir_type == 'diviceDate_device':
            dir_path = os.path.join(dir_path, self.top_dir_name)
            dir_path = os.path.join(dir_path, self.timestamp.strftime('%Y/%m/%d'))
            dir_path = os.path.join(dir_path, self.device_id)
        elif self.dir_type == 'diviceDateTime_device':
            dir_path = os.path.join(dir_path, self.top_dir_name)
            dir_path = os.path.join(dir_path, self.timestamp.strftime('%Y/%m/%d/%H/%M'))
            dir_path = os.path.join(dir_path, self.device_id)
        elif self.dir_type == 'device_divideDateTime':
            dir_path = os.path.join(dir_path, self.top_dir_name)
            dir_path = os.path.join(dir_path, self.device_id)
            dir_path = os.path.join(dir_path, self.timestamp.strftime('%Y/%m/%d/%H/%M'))
        elif self.dir_type == 'dateTime_device':
            dir_path = os.path.join(dir_path, self.top_dir_name)
            dir_path = os.path.join(dir_path, self.timestamp.strftime('%Y%m%dT%H-%M'))
            dir_path = os.path.join(dir_path, self.device_id)
        elif self.dir_type == 'device_dateTime':
            dir_path = os.path.join(dir_path, self.top_dir_name)
            dir_path = os.path.join(dir_path, self.device_id)
            dir_path = os.path.join(dir_path, self.timestamp.strftime('%Y%m%dT%H-%M'))
        return dir_path

    def getFileName(self, attr=None):
        filename = ''
        filename += self.filename_prefix
        if self.filename_type == 'default' or self.filename_type == 'device_dateTime_ms':
            filename += self.device_id
            filename += '_'
            filename += self.timestamp.strftime('%Y%m%dT%H-%M-%S-')+str(self.timestamp.microsecond)
        elif self.filename_type == 'device_dateTime':
            filename += self.device_id
            filename += '_'
            filename += self.timestamp.strftime('%Y%m%dT%H-%M-%S')
        if self.filename_suffix is not None:
            filename += self.filename_suffix
        if attr is not None:
            filename += attr
        if self.extention is not None and len(self.extention) > 0:
            filename += '.'
            filename += self.extention
        return filename

    def getFullPath(self, attr=None, timestamp=time.time()):
        date_time = datetime.datetime.fromtimestamp(timestamp)
        dir_path = self.getDirPath(date_time)
        filename = self.getFileName(attr=attr)
        path = os.path.join(dir_path, filename)
        return path
