# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_data_upload_thread_controller.py
# Copyright       : 2015-2020 SORACOM, INC.
# ======================================================


from time import sleep
from logging import getLogger
from soracom_data_upload_thread import SoracomDataUploadThread

logger = getLogger(__name__)

class SoracomDataUploadThreadController:
    def __init__(self, jobs, useMultiprocessing=True,
                 uploadDataPoolModule=None, fifo=True, multiprocessingTimeout=10):
        self.data_uploads_jobs = jobs
        self.data_uploads_works_index = 0
        self.data_uploads_working = 0
        self.data_uploads = []
        for i in range(self.data_uploads_jobs):
            self.data_uploads.append(SoracomDataUploadThread(
                useMultiprocessing, uploadDataPoolModule, multiprocessingTimeout))
        self.fifo = fifo

    def getResult(self):
        is_working = True
        result = None
        index = (self.data_uploads_works_index - self.data_uploads_working) % \
                self.data_uploads_jobs
        if self.data_uploads_working >= self.data_uploads_jobs:
            logger.debug('data_uploads[%d] data_uploads_works_index: %d data_uploads_working:%d',\
                        index, self.data_uploads_works_index, self.data_uploads_working)
            while is_working is True:
                is_working = self.data_uploads[index].getWorking()
                if is_working and not self.fifo:
                    index = (index + 1) % self.data_uploads_jobs
                sleep(0.01)
            result = self.data_uploads[index].getResult()
            self.data_uploads[index].resetUpload()
            self.data_uploads_working = \
                self.data_uploads_working - 1 if self.data_uploads_working > 0 else 0
        logger.debug('get result of index: %d, data_uploads_works_index: %d \
            data_uploads_working: %d result: %s',\
                      index, self.data_uploads_works_index, self.data_uploads_working, result)
        return result

    def setData(self, device_id, timestamp, data_type, images,
                data, data_id=-1, resources=None, options=None):
        logger.debug('setData start device_id: %s timestamp: %s data_id: %d \
            data_uploads_works_index: %d data_uploads_working: %d', \
                device_id, str(timestamp), data_id, self.data_uploads_works_index, \
                    self.data_uploads_working)
        is_working = True
        index = (self.data_uploads_works_index -
                 self.data_uploads_working) % self.data_uploads_jobs
        if self.data_uploads_working >= self.data_uploads_jobs or not self.fifo:
            logger.debug('data_uploads[%d]', index)
            while is_working is True:
                is_working = self.data_uploads[index].getWorking()
                if is_working and not self.fifo:
                    index = (index + 1) % self.data_uploads_jobs
                sleep(0.01)
        if self.fifo:
            index = self.data_uploads_works_index % self.data_uploads_jobs
        self.data_uploads[index].startUpload(
            device_id, timestamp, data_type, images, data, data_id, resources, options)
        self.data_uploads_works_index += 1
        self.data_uploads_works_index %= self.data_uploads_jobs * 2
        self.data_uploads_working += 1
        logger.debug('data_uploads_works_index: %d, data_uploads_working: %d data_id: %d',
                     self.data_uploads_works_index, self.data_uploads_working, data_id)
        logger.debug('setData end')


    def putData(self, device_id, timestamp, data_type, images, data,
                data_id=-1, resources=None, options=None):
        result = self.getResult()
        self.setData(device_id, timestamp, data_type, images, data, data_id, resources, options)
        return result
