# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_hls_util.py
# Copyright       : 2015-2021 SORACOM, INC.
# ======================================================

import os
import logging
from logging import getLogger
import requests
import time

from soracom_debug_util import SoracomDummyResponse

logging.basicConfig(level=logging.DEBUG)
logger = getLogger(__name__)

class SoracomHlsUtil:
    def __init__(self):
        self.scheme = 'http'
        self.host = 'localhost'
        self.port = 8086
        self.start_time = 0
        self.end_time = 0


    def getM3U8List(self, s_time, e_time=time.time(), debug=False):
        "get m3u8 list"

        start_time, end_time = self.duration(s_time, e_time)
        logger.debug("start_time: %d end_time: %d", start_time, end_time)
        path = os.path.join("v1/m3u8/", str(int(start_time)) + "/" + str(int(end_time)))
        path = os.path.join(path, "index.m3u8")
        try:
            url = self.scheme
            url += "://"
            url += self.host
            if self.port > 0:
                url += ":"
                url += str(self.port)
            url += "/"
            url += path
            logger.info('url: %s', url)
            if debug:
                return SoracomDummyResponse()
            response = requests.get(url)
            logger.debug('response from Local Harvest Agent: %s', response)
            return response.text
        except Exception as err:
            logger.error("can't upload data by %s", err)
            return None

    def duration(self, start_time, end_time):
        if self.start_time > start_time:
            start_time = self.start_time
        if self.end_time < end_time:
            self.end_time = end_time
        if self.end_time > self.start_time:
            self.start_time = self.end_time

        return start_time, end_time

    def getTsFiles(self, m3u8list, start_time, end_time=time.time()):
        path = "v1/m3u8/" + str(int(start_time)) + "/" + str(int(end_time))
        tsfiles = m3u8list.split('\n')
        download_list = []
        url = self.scheme
        url += "://"
        url += self.host
        if self.port > 0:
            url += ":"
            url += str(self.port)
        url += "/"
        url += path + "/"
        for line in tsfiles:
            if line.startswith("ts/"):
                download_list.append(url+line)
                logger.info('ts file url: %s', url)
        return download_list
