# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_mosaic_coordinate.py
# Copyright       : 2015-2022 SORACOM, INC.
# ======================================================

import os
import json

class SoracomMosaicCoordinate:
    has_mosaic_coordinate = False
    mosaic_coordinate = None

    def __init__(self, free_param_object):
        if "mosaic_coordinate" in free_param_object:
            self.mosaic_coordinate = free_param_object["mosaic_coordinate"]
            if self.mosaic_coordinate:
                self.has_mosaic_coordinate = True
        print( "has_mosaic_coordinate", self.has_mosaic_coordinate )

    def maked_camera_size(self):
        if not self.has_mosaic_coordinate:
            return None
        image_size = self.mosaic_coordinate["info"]["ImageSize"]
        #print( imageSize )
        return image_size

    def line_count(self):
        if not self.has_mosaic_coordinate:
            return 0
        if "lines" not in self.mosaic_coordinate:
            return 0
        lines = self.mosaic_coordinate["lines"]
        #print( lines )
        return len(lines)

    def line_no(self, list_no):
        if not self.has_mosaic_coordinate:
            return None
        if "lines" not in self.mosaic_coordinate:
            return None
        lines = self.mosaic_coordinate["lines"]
        if len(lines) <= list_no:
            return None
        return lines[list_no]

    def line_name(self, name):
        if not self.has_mosaic_coordinate:
            return None
        if "lines" not in self.mosaic_coordinate:
            return None
        lines = self.mosaic_coordinate["lines"]
        for line in lines:
            if line["name"] == name:
                return line
        return None

    def rect_count(self):
        if not self.has_mosaic_coordinate:
            return None
        if "rects" not in self.mosaic_coordinate:
            return 0
        rects = self.mosaic_coordinate["rects"]
        #print( rects )
        return len(rects)

    def rect_no(self,list_no):
        if not self.has_mosaic_coordinate:
            return None
        if "rects" not in self.mosaic_coordinate:
            return None
        rects = self.mosaic_coordinate["rects"]
        if len(rects) <= list_no:
            return None
        #print( rects )
        return rects[list_no]

    def rect_name(self, name):
        if not self.has_mosaic_coordinate:
            return None
        if "rects" not in self.mosaic_coordinate:
            return None
        rects = self.mosaic_coordinate["rects"]
        for rect in rects:
            if rect["name"] == name:
                return rect
        return None

    def max_boundary(self, src_rect):
        locs = src_rect["loc"]

        x_min = locs[0]["x"]
        x_max = locs[0]["x"]
        y_min = locs[0]["y"]
        y_max = locs[0]["y"]

        for loc in locs:
            if loc["x"] < x_min:
                x_min = loc["x"]
            if loc["x"] > x_max:
                x_max = loc["x"]
            if loc["y"] < y_min:
                y_min = loc["y"]
            if loc["y"] > y_max:
                y_max = loc["y"]

        boundary =  {"x_min": x_min, "x_max": x_max, "y_min": y_min, "y_max": y_max}
        return boundary

    def max_boundary_rect_no(self, list_no):
        rect = self.rect_no(list_no)
        if rect is None:
            return None
        return self.max_boundary( rect )

    def max_boundary_rect_name(self, name):
        rect = self.rect_no(name)
        if rect is None:
            return None
        return self.max_boundary( rect )

    def get_image_size(self):
        if not self.has_mosaic_coordinate:
            return None
        if "info" not in self.mosaic_coordinate:
            return None
        info = self.mosaic_coordinate["info"]
        if "ImageSize" not in info:
            return None
        image_size = info["ImageSize"]
        return image_size

